# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License.


# This is testing the function load feature for the single event annotation
# The event shouldn't be int
def main(event: int) -> str:
    return 'BAD'
