# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License.


"""This module pytest is provided inside customer's code,
used for checking module name collision"""
__version__ = 'from.customer.code'
